@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/main.c,v 1.9 2005/11/09 14:12:21 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/main.c,v 1.16 2007/02/01 15:31:59 yamaguti Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2005	Tatsuo Ishii
+ * Copyright (c) 2003-2007	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -87,6 +87,9 @@ static int switch_over_sig = SIGUSR1;	/* switch over signal default value */
 
 static int health_check_timer_expired;		/* non 0 if health check timer expired */
 
+int myargc;
+char **myargv;
+
 /*
 * pgpool main program
 */
@@ -94,15 +97,29 @@ int main(int argc, char **argv)
 {
 	int opt;
 	char conf_file[POOLMAXPATHLEN+1];
+	char hba_file[POOLMAXPATHLEN+1];
 	int i;
 	int pid;
 
-	snprintf(conf_file, sizeof(conf_file), "%s/%s", DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
+	myargc = argc;
+	myargv = argv;
 
-	while ((opt = getopt(argc, argv, "df:hm:ns:")) != -1)
+	snprintf(conf_file, sizeof(conf_file), "%s/%s", DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
+	snprintf(hba_file, sizeof(hba_file), "%s/%s", DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
+
+	while ((opt = getopt(argc, argv, "a:df:hm:ns:")) != -1)
 	{
 		switch (opt)
 		{
+			case 'a':    /* specify hba configuration file */
+				if (!optarg)
+				{
+					usage();
+					exit(1);
+				}
+				strncpy(hba_file, optarg, sizeof(hba_file));
+				break;
+
 			case 'd':	/* debug option */
 				debug = 1;
 				break;
@@ -170,8 +187,7 @@ int main(int argc, char **argv)
 	if (pool_get_config(conf_file))
 	{
 		pool_error("Unable to get configuration. Exiting...");
-		mypid = 0;		/* do not delete UNIX domain socket or pid file */
-		myexit(1);
+		exit(1);
 	}
 
 	/* set current PostgreSQL backend */
@@ -179,10 +195,18 @@ int main(int argc, char **argv)
 	pool_config.current_backend_port = pool_config.backend_port;
 
 	/* set load balance weight */
-	weight_master = (RAND_MAX) * (pool_config.weight_master /
-						   (pool_config.weight_master + pool_config.weight_secondary));
+	if (pool_config.weight_master <= 0.0)
+		weight_master = 0;
+	else
+		weight_master = (RAND_MAX) * (pool_config.weight_master /
+									  (pool_config.weight_master + pool_config.weight_secondary));
+
 	pool_debug("weight: %ld", weight_master);
 
+	/* read pool_hba.conf */
+	if (pool_config.enable_pool_hba)
+		load_hba(hba_file);
+
 	/*
 	 * if a non-switch argument remains, then it should be either "stop" or "switch"
 	 */
@@ -349,11 +373,12 @@ static void usage(void)
 {
 	fprintf(stderr, "pgpool version %s(%s),\n",	VERSION, PGPOOLVERSION);
 	fprintf(stderr, "  a generic connection pool/replication/load balance server for PostgreSQL\n\n");
-	fprintf(stderr, "usage: pgpool [-f config_file][-n][-d]\n");
-	fprintf(stderr, "usage: pgpool [-f config_file] [-m {s[mart]|f[ast]|i[mmediate]}] stop\n");
-	fprintf(stderr, "usage: pgpool [-f config_file] [-s {m[aster]|s[econdary]] switch\n");
+	fprintf(stderr, "usage: pgpool [-f config_file][-a hba_file][-n][-d]\n");
+	fprintf(stderr, "usage: pgpool [-f config_file][-a hba_file] [-m {s[mart]|f[ast]|i[mmediate]}] stop\n");
+	fprintf(stderr, "usage: pgpool [-f config_file][-a hba_file] [-s {m[aster]|s[econdary]] switch\n");
 	fprintf(stderr, "usage: pgpool -h\n");
 	fprintf(stderr, "  config_file default path: %s/%s\n",DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
+	fprintf(stderr, "  hba_file default path:    %s/%s\n",DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
 	fprintf(stderr, "  -n: don't run in daemon mode. does not detatch control tty\n");
 	fprintf(stderr, "  -d: debug mode. lots of debug information will be printed\n");
 	fprintf(stderr, "  stop: stop pgpool\n");
@@ -509,7 +534,7 @@ static void write_pid_file(void)
 /*
 * fork a child
 */
-pid_t fork_a_child(int unix_fd, int inet_fd)
+static pid_t fork_a_child(int unix_fd, int inet_fd)
 {
 	pid_t pid;
 
@@ -517,6 +542,8 @@ pid_t fork_a_child(int unix_fd, int inet_fd)
 
 	if (pid == 0)
 	{
+		myargv = save_ps_display_args(myargc, myargv);
+
 		/* call child main */
 		POOL_SETMASK(&UnBlockSig);
 		do_child(unix_fd, inet_fd);
