@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool/pool.h,v 1.3 2005/06/25 00:58:46 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool.h,v 1.16 2007/02/03 04:22:21 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2005	Tatsuo Ishii
+ * Copyright (c) 2003-2007	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -28,8 +28,12 @@
 
 #include "config.h"
 #include "pool_signal.h"
+#include "pool_type.h"
+#include "pool_list.h"
+
 #include <stdio.h>
 #include <time.h>
+#include <netinet/in.h>
 
 /* undef this if you have problems with non blocking accept() */
 #define NONE_BLOCK
@@ -38,6 +42,7 @@
 
 /* configuration file name */
 #define POOL_CONF_FILE_NAME "pgpool.conf"
+#define HBA_CONF_FILE_NAME "pool_hba.conf"
 
 /* pid file directory */
 #define DEFAULT_LOGDIR "/tmp"
@@ -119,6 +124,7 @@ typedef struct {
     int	num_init_children;	/* # of children initially pre-forked */
     int	child_life_time;	/* if idle for this seconds, child exits */
     int	connection_life_time;	/* if idle for this seconds, connection closes */
+    int	child_max_connections;	/* if max_connections received, child exits */
     int	max_pool;	/* max # of connection pool per child */
     char *logdir;		/* logging directory */
     char *backend_socket_dir;	/* Unix domain socket directory for the PostgreSQL server */
@@ -148,6 +154,7 @@ typedef struct {
 	char *health_check_user;		/* PostgreSQL user name for health check */
 	int insert_lock;	/* if non 0, automatically lock table with INSERT to keep SERIAL
 						   data consistency */
+	int ignore_leading_white_space;		/* ignore leading white spaces of each query */
 	/* followings do not exist in the configuration file */
     char *current_backend_host_name;	/* current backend host name */
     int	current_backend_port;	/* current backend port # */
@@ -156,6 +163,10 @@ typedef struct {
 	int num_reset_queries;		/* number of queries in reset_query_list */
 	int num_servers;			/* number of PostgreSQL servers */
 	int server_status[MAX_CONNECTION_SLOTS];	/* server status 0:unused, 1:up, 2:down */
+	int log_statement; /* 0:false, 1: true - logs all SQL statements */
+	int log_connections;		/* 0:false, 1:true - logs incoming connections */
+	int log_hostname;		/* 0:false, 1:true - resolve hostname */
+	int enable_pool_hba;		/* 0:false, 1:true - enables pool_hba.conf file authentication */
 } POOL_CONFIG;
 
 #define MAX_PASSWORD_SIZE		1024
@@ -171,7 +182,10 @@ typedef struct {
  */
 typedef struct {
 	int fd;		/* fd for connection */
-	FILE *write_fd;	/* stream write connection */
+
+	char *wbuf;	/* write buffer for the connection */
+	int wbufsz;	/* write buffer size */
+	int wbufpo;	/* buffer offset */
 
 	char *hp;	/* pending data buffer head address */
 	int po;		/* pending data offset */
@@ -205,6 +219,19 @@ typedef struct {
 
 	int no_forward;		/* if non 0, do not write to frontend */
 
+	/*
+	 * frontend info needed for hba
+	 */
+	int protoVersion;
+	SockAddr raddr;
+	UserAuth auth_method;
+	char *auth_arg;
+	char *database;
+	char *username;
+#ifdef USE_SSL
+	bool ssl;
+#endif
+
 } POOL_CONNECTION;
 
 /*
@@ -249,6 +276,7 @@ typedef struct {
 extern POOL_CONFIG pool_config;	/* configuration values */
 extern POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
 extern long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
+extern char remote_ps_data[];		/* used for set_ps_display */
 
 /*
  * public functions
@@ -261,7 +289,8 @@ extern void do_child(int unix_fd, int inet_fd);
 extern int pool_init_cp(void);
 extern POOL_STATUS pool_process_query(POOL_CONNECTION *frontend,
 									  POOL_CONNECTION_POOL *backend,
-									  int connection_reuse);
+									  int connection_reuse,
+									  int first_ready_for_query_received);
 
 extern POOL_CONNECTION *pool_open(int fd);
 extern void pool_close(POOL_CONNECTION *cp);
@@ -269,6 +298,7 @@ extern int pool_read(POOL_CONNECTION *cp, void *buf, int len);
 extern char *pool_read2(POOL_CONNECTION *cp, int len);
 extern int pool_write(POOL_CONNECTION *cp, void *buf, int len);
 extern int pool_flush(POOL_CONNECTION *cp);
+extern int pool_flush_it(POOL_CONNECTION *cp);
 extern int pool_write_and_flush(POOL_CONNECTION *cp, void *buf, int len);
 extern char *pool_read_string(POOL_CONNECTION *cp, int *len, int line);
 
@@ -282,6 +312,11 @@ extern void pool_discard_cp(char *user, char *database, int protoMajor);
 
 extern POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend, 
 								  POOL_CONNECTION_POOL *backend);
+extern POOL_STATUS ErrorResponse2(POOL_CONNECTION *frontend, 
+								  POOL_CONNECTION_POOL *backend);
+
+extern POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend, 
+								  POOL_CONNECTION_POOL *backend);
 
 extern void notice_backend_error(int master);
 
@@ -300,6 +335,7 @@ extern void pool_send_frontend_exits(POOL_CONNECTION_POOL *backend);
 extern int pool_read_message_length(POOL_CONNECTION_POOL *cp);
 extern int *pool_read_message_length2(POOL_CONNECTION_POOL *cp);
 extern signed char pool_read_kind(POOL_CONNECTION_POOL *cp);
+extern signed char pool_read_kind2(POOL_CONNECTION_POOL *cp);
 
 extern POOL_STATUS SimpleForwardToFrontend(char kind, POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 extern POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
@@ -323,4 +359,26 @@ extern void pool_send_error_message(POOL_CONNECTION *frontend, int protoMajor,
 extern void pool_free_startup_packet(StartupPacket *sp);
 extern int health_check(void);
 
+extern void init_prepared_list(void);
+
+extern size_t strlcpy(char *dst, const char *src, size_t siz);
+
+extern bool update_process_title;
+
+extern char **save_ps_display_args(int argc, char **argv);
+
+extern void init_ps_display(const char *username, const char *dbname,
+							const char *host_info, const char *initial_str);
+
+extern void set_ps_display(const char *activity, bool force);
+
+extern const char *get_ps_display(int *displen);
+
+/* pool_hba.c */
+extern void load_hba(char *hbapath);
+extern void ClientAuthentication(POOL_CONNECTION *frontend);
+
+/* pool_ip.c */
+extern void pool_getnameinfo_all(SockAddr *saddr, char *remote_host, char *remote_port);
+
 #endif /* POOL_H */
