@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_connection_pool.c,v 1.5 2006/06/26 08:25:17 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_connection_pool.c,v 1.7 2007/02/06 02:11:07 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2006	PgPool Global Development Group
+ * Copyright (c) 2003-2007	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -44,6 +44,7 @@ POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
 
 static POOL_CONNECTION_POOL_SLOT *create_cp(POOL_CONNECTION_POOL_SLOT *cp, int secondary_backend);
 static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p);
+static int check_socket_status(int fd);
 
 /*
 * initialize connection pools. this should be called once at the startup.
@@ -95,6 +96,24 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor)
 			/* mark this connection is under use */
 			MASTER_CONNECTION(p)->closetime = 0;
 			POOL_SETMASK(&oldmask);
+
+			if (check_socket_status(MASTER(p)->fd) < 0 ||
+				(DUAL_MODE && check_socket_status(MASTER(p)->fd) < 0))
+			{
+				pool_log("connection closed. retry to create new connection pool.");
+				pool_free_startup_packet(MASTER_CONNECTION(p)->sp);
+				pool_close(MASTER_CONNECTION(p)->con);
+
+				if (DUAL_MODE)
+				{
+					/* do not free memory! we did not allocate them */
+					pool_close(SECONDARY_CONNECTION(p)->con);
+				}
+
+				memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+				return NULL;
+			}
+
 			return p;
 		}
 		p++;
@@ -462,3 +481,34 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 
 	return p;
 }
+
+/* check_socket_status()
+ * RETURN: 0 => OK
+ *        -1 => broken socket.
+ */
+static int check_socket_status(int fd)
+{
+	fd_set rfds;
+	int result;
+	struct timeval t;
+
+	for (;;)
+	{
+		FD_ZERO(&rfds);
+		FD_SET(fd, &rfds);
+
+		t.tv_sec = t.tv_usec = 0;
+
+		result = select(fd+1, &rfds, NULL, NULL, &t);
+		if (result < 0 && errno == EINTR)
+		{
+			continue;
+		}
+		else
+		{
+			return (result == 0 ? 0 : -1);
+		}
+	}
+
+	return -1;
+}
